@@ -23,6 +23,8 @@
 #include <config.h>
 #endif
 
+#ifndef HAVE_W32_SYSTEM
+
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -36,7 +38,7 @@
 #include "util.h"
 #include "ops.h"
 #include "wait.h"
-#include "io.h"
+#include "priv-io.h"
 #include "sema.h"
 
 #include "assuan.h"
@@ -83,7 +85,7 @@ struct engine_gpgsm
     void *fnc_value;
     struct
     {
-      unsigned char *line;
+      char *line;
       int linesize;
       int linelen;
     } attic;
@@ -96,18 +98,11 @@ struct engine_gpgsm
 typedef struct engine_gpgsm *engine_gpgsm_t;
 
 
-static const char *
-gpgsm_get_version (void)
+static char *
+gpgsm_get_version (const char *file_name)
 {
-  static const char *gpgsm_version;
-  DEFINE_STATIC_LOCK (gpgsm_version_lock);
-
-  LOCK (gpgsm_version_lock);
-  if (!gpgsm_version)
-    gpgsm_version = _gpgme_get_program_version (_gpgme_get_gpgsm_path ());
-  UNLOCK (gpgsm_version_lock);
-
-  return gpgsm_version;
+  return _gpgme_get_program_version (file_name ? file_name
+				     : _gpgme_get_gpgsm_path ());
 }
 
 
@@ -320,11 +315,13 @@ gpgsm_release (void *engine)
 
 
 static gpgme_error_t
-gpgsm_new (void **engine, const char *lc_ctype, const char *lc_messages)
+gpgsm_new (void **engine, const char *file_name, const char *home_dir,
+	   const char *lc_ctype, const char *lc_messages)
 {
   gpgme_error_t err = 0;
   engine_gpgsm_t gpgsm;
-  char *argv[3];
+  const char *argv[5];
+  int argc;
   int fds[2];
   int child_fds[4];
   char *dft_display = NULL;
@@ -396,12 +393,19 @@ gpgsm_new (void **engine, const char *lc_ctype, const char *lc_messages)
   child_fds[2] = gpgsm->message_fd_server;
   child_fds[3] = -1;
 
-  argv[0] = "gpgsm";
-  argv[1] = "--server";
-  argv[2] = NULL;
+  argc = 0;
+  argv[argc++] = "gpgsm";
+  if (home_dir)
+    {
+      argv[argc++] = "--homedir";
+      argv[argc++] = home_dir;
+    }
+  argv[argc++] = "--server";
+  argv[argc++] = NULL;
 
   err = assuan_pipe_connect (&gpgsm->assuan_ctx,
-			     _gpgme_get_gpgsm_path (), argv, child_fds);
+			     file_name ? file_name : _gpgme_get_gpgsm_path (),
+			     argv, child_fds);
   /* FIXME: Check error.  */
 
   /* We need to know the fd used by assuan for reads.  We do this by
@@ -745,17 +749,16 @@ status_handler (void *opaque, int fd)
           /* FIXME We can't use this for binary data because we
              assume this is a string.  For the current usage of colon
              output it is correct.  */
-          unsigned char *src = line + 2;
-	  unsigned char *end = line + linelen;
-	  unsigned char *dst;
-          unsigned char **aline = &gpgsm->colon.attic.line;
+          char *src = line + 2;
+	  char *end = line + linelen;
+	  char *dst;
+          char **aline = &gpgsm->colon.attic.line;
 	  int *alinelen = &gpgsm->colon.attic.linelen;
 
 	  if (gpgsm->colon.attic.linesize
 	      < *alinelen + linelen + 1)
 	    {
-	      unsigned char *newline = realloc (*aline,
-						*alinelen + linelen + 1);
+	      char *newline = realloc (*aline, *alinelen + linelen + 1);
 	      if (!newline)
 		err = gpg_error_from_errno (errno);
 	      else
@@ -774,7 +777,7 @@ status_handler (void *opaque, int fd)
 		    {
 		      /* Handle escaped characters.  */
 		      ++src;
-		      *dst = (unsigned char) _gpgme_hextobyte (src);
+		      *dst = _gpgme_hextobyte (src);
 		      (*alinelen)++;
 		      src += 2;
 		    }
@@ -1602,3 +1605,5 @@ struct engine_ops _gpgme_engine_ops_gpgsm =
     gpgsm_io_event,
     gpgsm_cancel
   };
+
+#endif /*!HAVE_W32_SYSTEM*/
